﻿using Dapper;
using MySqlConnector;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ArchiverLite.Services
{
    public class MesLegacyService
    {
        private readonly MySqlConnection _db;

        public MesLegacyService(string connectionString)
        {
            var connBuilder = new MySqlConnectionStringBuilder(connectionString);
            connBuilder.SslMode = MySqlSslMode.None;
            connBuilder.CharacterSet = "utf8";
            //connBuilder.ConnectionTimeout = 5 * 60;
            _db = new MySqlConnection(connectionString);
        }

        public void ClearOrder(string starMonth)
        {
            foreach (var orderno in GetIdList<string>("saleorder", "orderno", "LEFT(DATE_FORMAT(SaleDate,'%Y%m'),6) < @starMonth", new { starMonth }))
            {
                using (var trans = GetTransaction())
                {
                    foreach (var childTable in new[] { "SaleOrderItem", "saleblock", "saleobject", "saleorderobject", "saleorderoffer", "saleorderpackage", "saleblockobjids", "saleobjectobjids", "saleorder_block_cadmodelinfo", "saleorderblockbaseposition",
                                 "saleordergoodsinfo", "saleorder_block_point", "saleorder_block_pointinfo", "orderprocess", "orderprocessstep", "orderprocessstepitem"})
                    {
                        DeleteRow(childTable, "orderno", new { orderno }, trans);
                    }
                    DeleteRow("saleorder", "orderno", new { orderno }, trans);
                    trans.Commit();
                }
            }
        }

        public void ClearOrderBlockPlan(string starMonth)
        {
            foreach (var id in GetIdList<string>("orderblockprocessplan", "id", "LEFT(DATE_FORMAT(createtime,'%Y%m'),6) < @starMonth", new { starMonth }))
            {
                using (var trans = GetTransaction())
                {
                    DeleteRow("orderblockprocessplan", "ID", new { ID = id }, trans);
                    DeleteRow("orderblockprocessplanplaceresult", "ID", new { ID = id }, trans);
                    trans.Commit();
                }
            }
        }

        private void DeleteRow(string table, string key, object param, MySqlTransaction? transaction)
        {
            _db.Execute($"DELETE FROM `{table}` where {key}=@{key}", param, transaction);
        }

        private IEnumerable<T> GetIdList<T>(string table, string field, string condition, object param)
        {
            return _db.Query<T>($"select {field} from `{table}` where {condition}", param);
        }

        private MySqlTransaction GetTransaction()
        {
            if (_db.State != System.Data.ConnectionState.Open) _db.Open();
            return _db.BeginTransaction();
        }
    }
}
